/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.dsl

import ch.qos.logback.classic.Level
import ch.qos.logback.classic.Logger
import metridoc.extension.ExtensionRunner
import org.slf4j.LoggerFactory

/**
 * Primary interface to run jobs.  Useful for embedding metridoc into standard java apps as well.  For
 * {@link JobBuilder} documentation see http://code.google.com/p/metridoc/wiki/JobBuilder, for documentation on job
 * files please see http://code.google.com/p/metridoc/wiki/JobFile
 *
 *
 */
class JobBuilder {

    def ExtensionRunner extensionRunner
    def Binding services = new Binding()
    def log = LoggerFactory.getLogger("${METRIDOC_JOBS}.builder")
    boolean stopCalled = false
    private static Set markedJobs = []

    public static final String DEFAULT_JOB = "defaultJob"
    public static final String METRIDOC_JOBS = "metridoc.jobs"
    public static final String METRIDOC_BUILDER = "${METRIDOC_JOBS}.builder"

    ExtensionRunner getExtensionRunner() {
        if (extensionRunner) {
            return extensionRunner
        }

        extensionRunner = new ExtensionRunner()
    }

    def static void run(LinkedHashMap args = null, Closure closure) {
        run(args, DEFAULT_JOB, closure)
    }

    def static void run(LinkedHashMap args = null, String jobName, Closure closure) {
        JobBuilder.run(args, [jobName] as String[], closure)
    }

    def static void run(LinkedHashMap args = null, String[] jobNames, Closure closure) {
        def builder = new JobBuilder()
        configureBuilder builder, args
        def jobRunner = new JobRunner(services: builder.services)
        builder.addPlugins(JobRunner)
        new JobsHelper(closure: closure, services: builder.services).run()
        jobRunner.runJobs jobNames
    }

    def static void jobs(LinkedHashMap args = null, Closure closure) {
        jobs(args, DEFAULT_JOB, closure)
    }

    def static void jobs(LinkedHashMap args = null, String jobName, Closure closure) {
        run(args, jobName, closure)
    }

    def static isJob(Object job) {
        if (markedJobs.contains(job)) return; //already declared it as a job
        markedJobs.add(job)

        def builder = getDefaultJobBuilder()
        job.metaClass.mixin JobRunner
        def jobRunner = job as JobRunner
        jobRunner.setServices(new Binding())
        if (job instanceof Script) {
            jobRunner.setServices(job.binding)
            job.out = System.out
        }

        def plugins = builder.getExtensionRunner().plugins
        if (plugins) {
            plugins.each {plugin ->
                job.metaClass.mixin(plugin)
            }
        }
        jobRunner.setPlugins(plugins)

        return job
    }

    def addPlugins(owner) {
        def plugins = getExtensionRunner().getPlugins()

        plugins.each {
            owner.metaClass.mixin it
        }
    }

    static JobBuilder getDefaultJobBuilder() {
        def defaultJobBuilder = new JobBuilder()
        configureBuilder(defaultJobBuilder, null)
        return defaultJobBuilder
    }

    static JobRunner job(Binding binding = null) {
        def builder = getDefaultJobBuilder()
        def jobRunner = new JobRunner()
        jobRunner.services = new Binding()

        if (binding) {
            binding.variables.each {key, value ->
                jobRunner.services.setVariable(key, value)
            }
        }

        def plugins = builder.getExtensionRunner().plugins
        if (plugins) {
            plugins.each {plugin ->
                jobRunner.metaClass.mixin(plugin)
            }
        }

        return jobRunner
    }

    private static void configureBuilder(JobBuilder builder, args) {
        builder.extensionRunner.binding.setVariable("jobBuilder", builder)
        builder.extensionRunner.runExtensions()
        if (args) {
            args.each {key, value ->
                builder.extensionRunner.binding.setProperty(key, value)
            }
        }
    }

}
